@@ -1,6 +1,6 @@
 /*
  * Unicode Conversion Library (Shift_JIS to UCS2)
- * 1999 by yoshidam
+ * 1999-2002 by yoshidam
  *
  */
 
@@ -17,7 +17,8 @@
 #include "ustring.h"
 
 #ifdef USE_WIN32API
-int s2u_conv2(const unsigned char* s, UString* u, unknown_sjis unknown_s_conv)
+int
+s2u_conv2(const unsigned char* s, UString* u, unknown_sjis unknown_s_conv)
 {
   int len = strlen((char*)s);
   int ulen;
@@ -34,7 +35,8 @@ int s2u_conv2(const unsigned char* s, UString* u, unknown_sjis unknown_s_conv)
   return u->len;
 }
 #else
-int s2u_conv2(const unsigned char* s, UString* u, unknown_sjis unknown_s_conv)
+int
+s2u_conv2(const unsigned char* s, UString* u, unknown_sjis unknown_s_conv)
 {
   int i;
   int len = strlen((char*)s);
@@ -69,8 +71,14 @@ int s2u_conv2(const unsigned char* s, UString* u, unknown_sjis unknown_s_conv)
 	sc = '?';
 	if (unknown_s_conv != NULL) {
 	  unsigned char us[3];
+	  VALUE ret;
 	  us[0] = s[i]; us[1] = s[i+1]; us[2] = 0;
-	  sc = unknown_s_conv(us);
+	  ret = unknown_s_conv(us);
+	  if (TYPE(ret) != T_FIXNUM) {
+	    UStr_free(u);
+	    rb_exc_raise(ret);
+	  }
+	  sc = FIX2INT(ret);
 	}
       }
       UStr_addChar2(u, sc & 0xff, sc >> 8);
@@ -79,9 +87,15 @@ int s2u_conv2(const unsigned char* s, UString* u, unknown_sjis unknown_s_conv)
     else { /* Illegal sequence */
       unsigned short sc = '?';
       if (unknown_s_conv != NULL) {
+	VALUE ret;
 	unsigned char us[2];
 	us[0] = s[i]; us[1] = 0;
-	sc = unknown_s_conv(us);
+	ret = unknown_s_conv(us);
+	if (TYPE(ret) != T_FIXNUM) {
+	  UStr_free(u);
+	  rb_exc_raise(ret);
+	}
+	sc = FIX2INT(ret);
       }
       UStr_addChar2(u, sc & 0xff, sc >> 8);
     }
@@ -91,21 +105,4 @@ int s2u_conv2(const unsigned char* s, UString* u, unknown_sjis unknown_s_conv)
 }
 #endif
 
-#if 0
-unsigned short
-conv(const unsigned char* s)
-{
-  return '?';
-}
-
-int
-main(void)
-{
-  UString u;
-  s2u_conv2("Abc\x82\xa0\x82\xa2\x82\xa4", &u, conv);
-  UStr_dump(&u);
-  return 0;
-}
-#endif
-
 #endif /* USE_SJIS */
