@@ -1,5 +1,6 @@
 /*
- * uconv version 0.4.10
+ * uconv version 0.4.12
+ * Mar 12, 2003 yoshidam  for ruby 1.8.0
  * Sep  4, 2002 yoshidam  fixes memory leaks
  * Dec 10, 2001 yoshidam  supports the tainted status
  * Nov 23, 2001 yoshidam  appends shortest_flag
@@ -627,7 +628,7 @@ static VALUE
 unknown_euc_handler(const unsigned char* seq)
 {
   return rb_rescue((VALUE (*)(void*))call_euc_handler, (int)seq,
-		   exception_handler, NULL);
+		   exception_handler, Qnil);
 }
 #endif /* USE_EUC */
 
@@ -650,7 +651,7 @@ static VALUE
 unknown_sjis_handler(const unsigned char* seq)
 {
   return rb_rescue((VALUE (*)(void*))call_sjis_handler, (int)seq,
-		   exception_handler, NULL);
+		   exception_handler, Qnil);
 }
 #endif /* USE_SJIS */
 
@@ -672,7 +673,7 @@ static VALUE
 unknown_unicode_handler(unsigned short code)
 {
   return rb_rescue((VALUE (*)(void*))call_unicode_handler, code,
-		   exception_handler, NULL);
+		   exception_handler, Qnil);
 }
 
 #ifdef USE_EUC
@@ -688,10 +689,14 @@ uconv_u2toeuc(VALUE obj, VALUE wstr)
 
   u = (unsigned char*)(RSTRING(wstr)->ptr);
   len = RSTRING(wstr)->len;
-
-  u2e_conv2(u, len, &e, unknown_unicode_handler);
-  ret = rb_str_new((char*)e.str, e.len);
-  UStr_free(&e);
+  if (u) {
+    u2e_conv2(u, len, &e, unknown_unicode_handler);
+    ret = rb_str_new((char*)e.str, e.len);
+    UStr_free(&e);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, wstr);
 
   return ret;
@@ -708,10 +713,15 @@ uconv_euctou2(VALUE obj, VALUE estr)
   Check_Type(estr, T_STRING);
 
   e = (unsigned char*)(RSTRING(estr)->ptr);
-  len = e2u_conv2(e, &u, unknown_euc_handler);
+  if (e) {
+    len = e2u_conv2(e, &u, unknown_euc_handler);
 
-  ret = rb_str_new((char*)u.str, u.len);
-  UStr_free(&u);
+    ret = rb_str_new((char*)u.str, u.len);
+    UStr_free(&u);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, estr);
 
   return ret;
@@ -730,11 +740,16 @@ uconv_u2tosjis(VALUE obj, VALUE wstr)
   Check_Type(wstr, T_STRING);
 
   u = (unsigned char*)(RSTRING(wstr)->ptr);
-  len = RSTRING(wstr)->len;
+  if (u) {
+    len = RSTRING(wstr)->len;
 
-  u2s_conv2(u, len, &s, unknown_unicode_handler);
-  ret = rb_str_new((char*)s.str, s.len);
-  UStr_free(&s);
+    u2s_conv2(u, len, &s, unknown_unicode_handler);
+    ret = rb_str_new((char*)s.str, s.len);
+    UStr_free(&s);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, wstr);
 
   return ret;
@@ -751,10 +766,15 @@ uconv_sjistou2(VALUE obj, VALUE sstr)
   Check_Type(sstr, T_STRING);
 
   s = (unsigned char*)(RSTRING(sstr)->ptr);
-  len = s2u_conv2(s, &u, unknown_sjis_handler);
+  if (s) {
+    len = s2u_conv2(s, &u, unknown_sjis_handler);
 
-  ret = rb_str_new((char*)u.str, u.len);
-  UStr_free(&u);
+    ret = rb_str_new((char*)u.str, u.len);
+    UStr_free(&u);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, sstr);
 
   return ret;
@@ -774,7 +794,7 @@ uconv_u2swap(VALUE obj, VALUE wstr)
 
   u = (unsigned char*)(RSTRING(wstr)->ptr);
   len = RSTRING(wstr)->len;
-  if (len < 2) return Qnil;
+  if (!u || len < 2) return Qnil;
   ret = rb_str_new(NULL, len);
   out = (unsigned char*)(RSTRING(ret)->ptr);
 
@@ -799,7 +819,7 @@ uconv_u2swap_b(VALUE obj, VALUE wstr)
   rb_str_modify(wstr);
   u = (unsigned char*)(RSTRING(wstr)->ptr);
   len = RSTRING(wstr)->len;
-  if (len < 2) return Qnil;
+  if (!u || len < 2) return Qnil;
 
   for (i = 0; i < len; i+=2) {
     register unsigned char tmp = u[i+1];
@@ -822,7 +842,7 @@ uconv_u4swap(VALUE obj, VALUE wstr)
 
   u = (unsigned char*)(RSTRING(wstr)->ptr);
   len = RSTRING(wstr)->len;
-  if (len < 4) return Qnil;
+  if (!u || len < 4) return Qnil;
   ret = rb_str_new(NULL, len);
   out = (unsigned char*)(RSTRING(ret)->ptr);
 
@@ -872,9 +892,14 @@ uconv_u8tou16(VALUE obj, VALUE ustr)
 
   Check_Type(ustr, T_STRING);
   in = (unsigned char*)(RSTRING(ustr)->ptr);
-  _u8tou16(in, &out);
-  ret = rb_str_new((char*)out.str, out.len);
-  UStr_free(&out);
+  if (in) {
+    _u8tou16(in, &out);
+    ret = rb_str_new((char*)out.str, out.len);
+    UStr_free(&out);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, ustr);
 
   return ret;
@@ -890,10 +915,15 @@ uconv_u16tou8(VALUE obj, VALUE wstr)
 
   Check_Type(wstr, T_STRING);
   in = (unsigned char*)(RSTRING(wstr)->ptr);
-  len = RSTRING(wstr)->len;
-  _u16tou8(in, len, &out, eliminate_zwnbsp_flag);
-  ret = rb_str_new((char*)out.str, out.len);
-  UStr_free(&out);
+  if (in) {
+    len = RSTRING(wstr)->len;
+    _u16tou8(in, len, &out, eliminate_zwnbsp_flag);
+    ret = rb_str_new((char*)out.str, out.len);
+    UStr_free(&out);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, wstr);
 
   return ret;
@@ -909,9 +939,14 @@ uconv_u8tou4(VALUE obj, VALUE ustr)
 
   Check_Type(ustr, T_STRING);
   in = (unsigned char*)(RSTRING(ustr)->ptr);
-  _u8tou4(in, &out);
-  ret = rb_str_new((char*)out.str, out.len);
-  UStr_free(&out);
+  if (in) {
+    _u8tou4(in, &out);
+    ret = rb_str_new((char*)out.str, out.len);
+    UStr_free(&out);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, ustr);
 
   return ret;
@@ -927,10 +962,15 @@ uconv_u4tou8(VALUE obj, VALUE wstr)
 
   Check_Type(wstr, T_STRING);
   in = (unsigned char*)(RSTRING(wstr)->ptr);
-  len = RSTRING(wstr)->len;
-  _u4tou8(in, len, &out, eliminate_zwnbsp_flag);
-  ret = rb_str_new((char*)out.str, out.len);
-  UStr_free(&out);
+  if (in) {
+    len = RSTRING(wstr)->len;
+    _u4tou8(in, len, &out, eliminate_zwnbsp_flag);
+    ret = rb_str_new((char*)out.str, out.len);
+    UStr_free(&out);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, wstr);
 
   return ret;
@@ -947,10 +987,15 @@ uconv_u16tou4(VALUE obj, VALUE wstr)
 
   Check_Type(wstr, T_STRING);
   in = (unsigned char*)(RSTRING(wstr)->ptr);
-  len = RSTRING(wstr)->len;
-  _u16tou4(in, len, &out);
-  ret = rb_str_new((char*)out.str, out.len);
-  UStr_free(&out);
+  if (in) {
+    len = RSTRING(wstr)->len;
+    _u16tou4(in, len, &out);
+    ret = rb_str_new((char*)out.str, out.len);
+    UStr_free(&out);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, wstr);
 
   return ret;
@@ -966,10 +1011,15 @@ uconv_u4tou16(VALUE obj, VALUE wstr)
 
   Check_Type(wstr, T_STRING);
   in = (unsigned char*)(RSTRING(wstr)->ptr);
-  len = RSTRING(wstr)->len;
-  _u4tou16(in, len, &out);
-  ret = rb_str_new((char*)out.str, out.len);
-  UStr_free(&out);
+  if (in) {
+    len = RSTRING(wstr)->len;
+    _u4tou16(in, len, &out);
+    ret = rb_str_new((char*)out.str, out.len);
+    UStr_free(&out);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, wstr);
 
   return ret;
@@ -987,13 +1037,18 @@ uconv_u8toeuc(VALUE obj, VALUE ustr)
 
   Check_Type(ustr, T_STRING);
   in = (unsigned char*)(RSTRING(ustr)->ptr);
-  _u8tou16(in, &out);
-  vout = rb_str_new((char*)out.str, out.len);
-  UStr_free(&out);
-  u2e_conv2(RSTRING(vout)->ptr, RSTRING(vout)->len,
-	    &e, unknown_unicode_handler);
-  ret = rb_str_new((char*)e.str, e.len);
-  UStr_free(&e);
+  if (in) {
+    _u8tou16(in, &out);
+    vout = rb_str_new((char*)out.str, out.len);
+    UStr_free(&out);
+    u2e_conv2(RSTRING(vout)->ptr, RSTRING(vout)->len,
+	      &e, unknown_unicode_handler);
+    ret = rb_str_new((char*)e.str, e.len);
+    UStr_free(&e);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, ustr);
 
   return ret;
@@ -1002,7 +1057,6 @@ uconv_u8toeuc(VALUE obj, VALUE ustr)
 static VALUE
 uconv_euctou8(VALUE obj, VALUE estr)
 {
-  int len;
   unsigned char* e;
   UString in;
   UString out;
@@ -1011,13 +1065,17 @@ uconv_euctou8(VALUE obj, VALUE estr)
 
   Check_Type(estr, T_STRING);
   e = (unsigned char*)(RSTRING(estr)->ptr);
-  len = RSTRING(estr)->len;
-  e2u_conv2(e, &in, unknown_euc_handler);
-  vin = rb_str_new((char*)in.str, in.len);
-  UStr_free(&in);
-  _u16tou8(RSTRING(vin)->ptr, RSTRING(vin)->len, &out, 1);
-  ret = rb_str_new((char*)out.str, out.len);
-  UStr_free(&out);
+  if (e) {
+    e2u_conv2(e, &in, unknown_euc_handler);
+    vin = rb_str_new((char*)in.str, in.len);
+    UStr_free(&in);
+    _u16tou8(RSTRING(vin)->ptr, RSTRING(vin)->len, &out, 1);
+    ret = rb_str_new((char*)out.str, out.len);
+    UStr_free(&out);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, estr);
 
   return ret;
@@ -1036,13 +1094,18 @@ uconv_u8tosjis(VALUE obj, VALUE ustr)
 
   Check_Type(ustr, T_STRING);
   in = (unsigned char*)(RSTRING(ustr)->ptr);
-  _u8tou16(in, &out);
-  vout = rb_str_new((char*)out.str, out.len);
-  UStr_free(&out);
-  u2s_conv2(RSTRING(vout)->ptr, RSTRING(vout)->len,
-	    &s, unknown_unicode_handler);
-  ret = rb_str_new((char*)s.str, s.len);
-  UStr_free(&s);
+  if (in) {
+    _u8tou16(in, &out);
+    vout = rb_str_new((char*)out.str, out.len);
+    UStr_free(&out);
+    u2s_conv2(RSTRING(vout)->ptr, RSTRING(vout)->len,
+	      &s, unknown_unicode_handler);
+    ret = rb_str_new((char*)s.str, s.len);
+    UStr_free(&s);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, ustr);
 
   return ret;
@@ -1051,7 +1114,6 @@ uconv_u8tosjis(VALUE obj, VALUE ustr)
 static VALUE
 uconv_sjistou8(VALUE obj, VALUE sstr)
 {
-  int len;
   unsigned char* s;
   UString in;
   UString out;
@@ -1060,13 +1122,17 @@ uconv_sjistou8(VALUE obj, VALUE sstr)
 
   Check_Type(sstr, T_STRING);
   s = (unsigned char*)(RSTRING(sstr)->ptr);
-  len = RSTRING(sstr)->len;
-  s2u_conv2(s, &in, unknown_sjis_handler);
-  vin = rb_str_new((char*)in.str, in.len);
-  UStr_free(&in);
-  _u16tou8(RSTRING(vin)->ptr, RSTRING(vin)->len, &out, 1);
-  ret = rb_str_new((char*)out.str, out.len);
-  UStr_free(&out);
+  if (s) {
+    s2u_conv2(s, &in, unknown_sjis_handler);
+    vin = rb_str_new((char*)in.str, in.len);
+    UStr_free(&in);
+    _u16tou8(RSTRING(vin)->ptr, RSTRING(vin)->len, &out, 1);
+    ret = rb_str_new((char*)out.str, out.len);
+    UStr_free(&out);
+  }
+  else {
+    ret = rb_str_new(NULL, 0);
+  }
   OBJ_INFECT(ret, sstr);
 
   return ret;
